# proof 1 sin high dataavailability
import matplotlib
matplotlib.use('TkAgg')
import matplotlib.pyplot as plt
# plt.rcParams['pgf.texsystem'] = 'pdflatex'
import itertools
from datastuff.data import generate_data_classification_twomoons, \
    generate_data_classification_twomoons_OoD, \
    generate_data_classification_stripes_v2, \
    generate_data_classification_stripes_v2_OoD
import torch
import torch.nn as nn
import torch.nn.functional as ff
from torch.optim.adam import Adam
import numpy as np
from helpers.batch_indexer import BatchIndexerShuffle, BatchIndexerSeq
from helpers.mysigma import varsig_from_logits, stdsig_from_logits
from helpers.aucroc import aucroc_multiclass
from helpers.calibplot import sliding_window_calib_curve
from copy import deepcopy
from scipy.stats.kde import gaussian_kde
from mpl_toolkits.axes_grid1 import make_axes_locatable
from pathlib import Path
from matplotlib.ticker import FormatStrFormatter
from sklearn.metrics import precision_recall_curve, auc, roc_auc_score

device = torch.device('cpu')


class Net(nn.Module):
    def __init__(self,
                 in_size: int,
                 num_layers: int,
                 hidden_size: int):
        super(Net, self).__init__()

        assert num_layers >= 1

        # first default layer
        self.linlay_0 = nn.Linear(in_size, hidden_size)
        self.relu_0 = nn.ReLU()

        # main model body
        self.modelbody = nn.Sequential()
        for i in range(num_layers - 1):
            self.modelbody.append(nn.Linear(hidden_size, hidden_size))
            self.modelbody.append(nn.ReLU())

        # out layer
        self.linlay_end = nn.Linear(hidden_size, 2)

    def forward(self, x):
        x = self.relu_0(self.linlay_0(x))
        x = self.modelbody(x)
        x = self.linlay_end(x)
        return x


################## DATA ############################
# data
dataset = "stripes"
if dataset == "stripes":
    x, y, x_eval, y_eval, x_test, y_test = generate_data_classification_stripes_v2()
if dataset == "two_moons":
    x, y, x_eval, y_eval, x_test, y_test = generate_data_classification_twomoons()


# OoD data
if dataset == "stripes":
    x_OoD, d_OoD_flat = generate_data_classification_stripes_v2_OoD()
if dataset == "two_moons":
    x_OoD, d_OoD_flat = generate_data_classification_twomoons_OoD()

# grid data
grid_shape_0 = 100
grid_shape_1 = 80
x0_min, x0_max, x1_min, x1_max = x[:, 0].min().item(), x[:, 0].max().item(), x[:, 1].min().item(), x[:, 1].max().item()
if dataset == "stripes":
    x0_test_vector = torch.linspace(x0_min - 3.0, x0_max + 3.0, grid_shape_0)
    x1_test_vector = torch.linspace(x1_min - 3.0, x1_max + 3.0, grid_shape_1)
if dataset == "two_moons":
    x0_test_vector = torch.linspace(x0_min - 1.0, x0_max + 1.0, grid_shape_0)
    x1_test_vector = torch.linspace(x1_min - 1.0, x1_max + 1.0, grid_shape_1)
x0_test_mg, x1_test_mg = torch.meshgrid(x0_test_vector, x1_test_vector)
x0_test_mg_flat = x0_test_mg.reshape(-1)
x1_test_mg_flat = x1_test_mg.reshape(-1)
x_test_mg_flat = torch.stack([x0_test_mg_flat, x1_test_mg_flat], dim=1)

################## TRAIN ############################

loss = ff.cross_entropy

x = x.to(device)
y = y.to(device)
x_eval = x_eval.to(device)
y_eval = y_eval.to(device)
x_test = x_test.to(device)
y_test = y_test.to(device)

num_layers = [1, 2, 3]
num_neurons = [100, 120, 150]
net_configurations = list(itertools.product(num_layers, num_neurons))
net_configurations = [item for item in net_configurations for _ in range(3)]

num_nets = len(net_configurations)

max_patience = 10
max_epochs = 8000

# for saving and later evaluating MI
net_predictions_mg_flat = []
net_predictions_test_flat = []
net_predictions_OoD_flat = []

# move to cuda
for i in range(num_nets):
    current_best_loss = torch.tensor(9999.0)
    current_best_model = None
    bidexer_train = BatchIndexerShuffle(len(x), None, 100, device, True)
    bidexer_eval = BatchIndexerSeq(len(x_eval), None, 100, device, False)
    net_config = net_configurations[i]
    net = Net(2, net_config[0], net_config[1])

    current_patience = 0
    interval = 5

    net = net.to(device)

    optim = Adam(net.parameters(), lr=0.001)

    net.train()

    for epoch in range(max_epochs):
        epoch_loss = 0
        for b_idxs in bidexer_train.generate_b_idx():
            b_inputs = x[b_idxs, :]
            b_targets = y[b_idxs]
            optim.zero_grad()
            pred = net(b_inputs)
            _loss = loss(pred, b_targets, reduction='mean', label_smoothing=0.001)
            epoch_loss += _loss.item()
            _loss.backward()
            optim.step()
            epoch_loss += _loss.item()
        if epoch % interval == 0:
            # do eval
            eval_loss = 0
            eval_preds = []
            eval_trgs = []
            with torch.no_grad():
                for b_idxs in bidexer_eval.generate_b_idx():
                    b_inputs = x_eval[b_idxs, :]
                    b_targets = y_eval[b_idxs]
                    pred = net(b_inputs)
                    _loss = loss(pred, b_targets, reduction='sum', label_smoothing=0.05)
                    eval_loss += _loss.item()
                    eval_preds.append(pred)
                    eval_trgs.append(b_targets)
            eval_preds = torch.concat(eval_preds, dim=0)
            eval_trgs = torch.concat(eval_trgs, dim=0)
            avg_score, _, _ = aucroc_multiclass(nn.functional.softmax(eval_preds, dim=-1).detach().cpu().numpy(),
                                                eval_trgs.view(-1).detach().cpu().numpy(),
                                                averaging='weighted')

            print('---------------------')
            print('EPOCH {}/{}'.format(epoch, max_epochs))
            print('EPOCH_LOSS: {}'.format(epoch_loss))
            print('EVAL_LOSS: {}'.format(eval_loss))
            print('AUCROC: {}'.format(avg_score))
            if eval_loss < current_best_loss:
                current_patience = 0
                current_best_loss = eval_loss
                current_best_model = net.state_dict()
            else:
                current_patience += 1
            print('PATIENCE {}/{}'.format(current_patience, max_patience))
            if current_patience >= max_patience:
                break

    ################## DATA COLLECTION ############################

    net.eval()

    # get normal pred for entire grid
    with torch.no_grad():
        x_test_mg_flat = x_test_mg_flat.to(device)
        pred = net(x_test_mg_flat)
        pred = pred.cpu()
    net_predictions_mg_flat.append(pred)

    # get predictions on test points defined trough the dataset
    with torch.no_grad():
        x_test = x_test.to(device)
        pred = net(x_test)
        pred = pred.cpu()
    net_predictions_test_flat.append(pred)

    # get predictions on OoD data
    with torch.no_grad():
        x_OoD = x_OoD.to(device)
        pred = net(x_OoD)
        pred = pred.cpu()
    net_predictions_OoD_flat.append(pred)

################## POSTPROC ############################

# for the grid
net_predictions_mg_flat = torch.stack(net_predictions_mg_flat, dim=1)
net_predictions_mg_flat_mean = torch.softmax(torch.mean(net_predictions_mg_flat, dim=1), dim=-1)[:, 0]

# for the test set
net_predictions_test_flat = torch.stack(net_predictions_test_flat, dim=1)
net_predictions_test_flat_mean = torch.softmax(torch.mean(net_predictions_test_flat, dim=1), dim=-1)
net_predictions_test_flat_mean_0 = net_predictions_test_flat_mean[:, 0]

# get the aucroc and aucpr measures
precision, recall, _ = precision_recall_curve(y_test.cpu().numpy(), net_predictions_test_flat_mean[:, 1])
aucpr = auc(recall, precision)
aucroc = roc_auc_score(y_test.cpu().numpy(), net_predictions_test_flat_mean[:, 1])
print("AUCROC: {}".format(aucroc))
print("AUCPR: {}".format(aucpr))

# for the OoD set
net_predictions_OoD_flat = torch.stack(net_predictions_OoD_flat, dim=1)
net_predictions_OoD_flat_mean = torch.softmax(torch.mean(net_predictions_OoD_flat, dim=1), dim=-1)
net_predictions_OoD_flat_mean_0 = net_predictions_OoD_flat_mean[:, 0]

################## MEASURES ############################

# calculate varsig for mg
varsigs_mg_flat = stdsig_from_logits(net_predictions_mg_flat)
varsigs_mg = varsigs_mg_flat.reshape(grid_shape_0, grid_shape_1).numpy()

# calculate varsig for OoD
varsigs_OoD_flat = stdsig_from_logits(net_predictions_OoD_flat).numpy().flatten()

# calculate varsig for test
varsigs_test_flat = stdsig_from_logits(net_predictions_test_flat).view(-1).numpy()

# calculate calibration
true_frac, pred_frac = sliding_window_calib_curve(net_predictions_test_flat_mean.cpu().numpy(), y_test.cpu().numpy(), bin_size=50)

################## PLOT ############################

x0_test_mg_flat = x0_test_mg_flat.numpy()
x1_test_mg_flat = x1_test_mg_flat.numpy()

net_predictions_mg_mean = net_predictions_mg_flat_mean.reshape(grid_shape_0, grid_shape_1).numpy()

fig_pred, ax_pred= plt.subplots()

# plot the actual prediction
prediction_image = ax_pred.imshow(np.transpose(net_predictions_mg_mean), vmin=net_predictions_mg_mean.min(), vmax=net_predictions_mg_mean.max(), origin='lower', interpolation='bilinear',
                                extent=[x0_test_mg_flat.min(), x0_test_mg_flat.max(),
                                        x1_test_mg_flat.min(), x1_test_mg_flat.max()])
# cbar_pred = plt.colorbar(prediction_image, ax=ax_pred, orientation='horizontal', shrink=0.75)
ax_pred.scatter(x[:, 0].view(-1).cpu().numpy(), x[:, 1].view(-1).cpu().numpy(), s=1, c="w", rasterized=True)

fig_varsig, ax_varsig= plt.subplots()

# run through log to show minute details in the lower ranges
varsigs_mg_transformed = np.log(varsigs_mg * 10 + 1)
varsigs_mg_transformed_scaled = (varsigs_mg_transformed - np.min(varsigs_mg_transformed)) / \
                                (np.max(varsigs_mg_transformed) - np.min(varsigs_mg_transformed))
uncertainty_image = ax_varsig.imshow(np.transpose(varsigs_mg_transformed), vmin=varsigs_mg_transformed.min(), vmax=varsigs_mg_transformed.max(), origin='lower', interpolation='bilinear',
                                     extent=[x0_test_mg_flat.min(), x0_test_mg_flat.max(),
                                             x1_test_mg_flat.min(), x1_test_mg_flat.max()])
# cbar_varsig = fig_varsig.colorbar(uncertainty_image, ax=ax_varsig, orientation='horizontal', shrink=0.75)
ax_varsig.scatter(x[:, 0].view(-1).cpu().numpy(), x[:, 1].view(-1).cpu().numpy(), s=1, c="w", rasterized=True)


# # calibration plot
# fig = plt.figure()
# ax = fig.subplots(1, 1)

# # plot the calibration
# ax.plot(pred_frac, true_frac)
# ax.plot(np.array([0, 1]), np.array([0, 1]))
# ax.set_xlabel("pred fraction")
# ax.set_ylabel("true fraction")
# plt.show(block=False)

# plot distance to uncertainty
fig_iidood_varsig, ax_iidood_varsig= plt.subplots()

# plot the actual prediction
varsigs_OoD_flat_transformed = np.log(varsigs_OoD_flat * 10 + 1)
varsigs_test_flat_transformed = np.log(varsigs_test_flat * 10 + 1)
ood_scatter = ax_iidood_varsig.scatter(d_OoD_flat.cpu().numpy(), varsigs_OoD_flat_transformed, s=1, rasterized=True)
if dataset == "two_moons":
    pdf_test = gaussian_kde(varsigs_test_flat_transformed, bw_method=0.2)
if dataset == "stripes":
    pdf_test = gaussian_kde(varsigs_test_flat_transformed, bw_method=0.2)
range_y = max(varsigs_test_flat_transformed) - min(varsigs_test_flat_transformed)
pdf_test_y = np.linspace(min(varsigs_test_flat_transformed)-range_y/4, max(varsigs_test_flat_transformed)+range_y/4, 600)
pdf_test_x = pdf_test(pdf_test_y)
pdf_test_x = - pdf_test_x / (4*max(pdf_test_x))
# clamp to 0 below 0 or delete
pdf_test_x = pdf_test_x[pdf_test_y >= 0.0]
pdf_test_y = pdf_test_y[pdf_test_y >= 0.0]
ax_iidood_varsig.plot(pdf_test_x, pdf_test_y, color="tab:orange")
pdf_density = ax_iidood_varsig.fill_betweenx(pdf_test_y, np.zeros_like(pdf_test_x), pdf_test_x, color="tab:orange", alpha=0.5)
cut_percentile = np.percentile(varsigs_test_flat_transformed, 90)
not_caught_shuffled = varsigs_OoD_flat_transformed < cut_percentile
print("not caught percent (VARSIG): {}".format(np.sum(not_caught_shuffled)/len(not_caught_shuffled)))
ax_iidood_varsig.plot([min(pdf_test_x), 1], [cut_percentile, cut_percentile], color="black")
ax_iidood_varsig.set_xlabel("distance")
ax_iidood_varsig.set_ylabel("uncertainty")

thresh_varsig_test = np.quantile(varsigs_test_flat_transformed, 0.9)
not_caught_varsig = varsigs_OoD_flat_transformed.flatten() < thresh_varsig_test
rejected_varsig = varsigs_test_flat_transformed > thresh_varsig_test
fig_not_caught_varsig, ax_not_caught_varsig= plt.subplots()
varsigs_mg_transformed = np.log(varsigs_mg * 10 + 1)
uncertainty_image = ax_not_caught_varsig.imshow(np.transpose(varsigs_mg_transformed), vmin=varsigs_mg_transformed.min(), vmax=varsigs_mg_transformed.max(), origin='lower', interpolation='bilinear',
                                 extent=[x0_test_mg_flat.min(), x0_test_mg_flat.max(),
                                         x1_test_mg_flat.min(), x1_test_mg_flat.max()])
# cbar_not_caught_varsig = fig_not_caught_varsig.colorbar(uncertainty_image, ax=ax_not_caught_varsig, orientation='horizontal', shrink=0.75)
ax_not_caught_varsig.scatter(x[:, 0].view(-1).cpu().numpy(), x[:, 1].view(-1).cpu().numpy(), s=1, c="w", marker='.', rasterized=True)
not_caught_scatter = ax_not_caught_varsig.scatter(x_OoD[not_caught_varsig, 0].view(-1).cpu().numpy(), x_OoD[not_caught_varsig, 1].view(-1).cpu().numpy(), s=12, c="r", marker='x', rasterized=True)


plt.show(block=False)
############## FORMATTING
cm2in = 1.0 / 2.54
mm2in = 1.0 / 25.4
single_height = 74
single_width = 74
double_width = 140
third_width = double_width/3
third_height = single_height * 0.66
scale_factor = 1
plt.rcParams['pgf.texsystem'] = 'xelatex'


x0_min, x0_max, x1_min, x1_max = np.round(x0_min, 1), np.round(x0_max, 1), np.round(x1_min, 1), np.round(x1_max, 1)
# ax_pred.set_xlabel('$x_1$', labelpad=-10)
# ax_pred.set_ylabel('$x_2$', labelpad=-17)
# ax_pred.set_aspect('equal', adjustable='box')
if dataset == "stripes":
    ax_pred.set_xlim([x0_min - 3.0, x0_max + 3.0])
    ax_pred.set_ylim([x1_min - 3.0, x1_max + 3.0])
if dataset == "two_moons":
    ax_pred.set_xlim([x0_min - 1.0, x0_max + 1.0])
    ax_pred.set_ylim([x1_min - 1.0, x1_max + 1.0])
# ax_pred.set_xticks([x0_min, x0_max])
# ax_pred.set_xticklabels([str(x0_min), str(x0_max)])
# ax_pred.set_yticks([x1_min, x1_max])
# ax_pred.set_yticklabels([str(x1_min), str(x1_max)])
ax_pred.set_xticks([])
ax_pred.set_xticklabels([])
ax_pred.set_yticks([])
ax_pred.set_yticklabels([])
vmin = np.ceil(net_predictions_mg_mean.min() * 10) / 10
vmax = np.floor(net_predictions_mg_mean.max() * 10) / 10
# cbar_pred.ax.get_xaxis().set_ticks([vmin, vmax])
# cbar_pred.ax.get_xaxis().labelpad = 5
# cbar_pred.set_label('$p(c_1)$')
# ax_pred.set_title('$p(c_0)$')
ax_pred.xaxis.set_major_formatter(FormatStrFormatter('%.1f'))
ax_pred.yaxis.set_major_formatter(FormatStrFormatter('%.1f'))
fig_pred.set_size_inches(mm2in * single_width / 2 * scale_factor, mm2in * single_height / 2 * scale_factor, forward=True)
fig_pred.subplots_adjust(
    top=0.90,
    bottom=0.15,
    left=0.15,
    right=0.97,
    hspace=0.05,
    wspace=0.05
)
# fig_pred.tight_layout(pad=0.2)
# fig_pred.tight_layout(pad=0.2)
# fig_pred.tight_layout(pad=0.2)
fig_pred.tight_layout(pad=0.1)
fig_pred.tight_layout(pad=0.1)
fig_pred.tight_layout(pad=0.1)

# ax_varsig.set_xlabel('$x_1$', labelpad=-10)
# ax_varsig.set_ylabel('$x_2$', labelpad=-17)
# ax_varsig.set_aspect('equal', adjustable='box')
if dataset == "stripes":
    ax_varsig.set_xlim([x0_min - 3.0, x0_max + 3.0])
    ax_varsig.set_ylim([x1_min - 3.0, x1_max + 3.0])
if dataset == "two_moons":
    ax_varsig.set_xlim([x0_min - 1.0, x0_max + 1.0])
    ax_varsig.set_ylim([x1_min - 1.0, x1_max + 1.0])
ax_varsig.set_xticks([])
ax_varsig.set_xticklabels([])
ax_varsig.set_yticks([])
ax_varsig.set_yticklabels([])
# ax_varsig.set_xticks([x0_min, x0_max])
# ax_varsig.set_xticklabels([str(x0_min), str(x0_max)])
# ax_varsig.set_yticks([x1_min, x1_max])
# ax_varsig.set_yticklabels([str(x1_min), str(x1_max)])
vmin = np.ceil(varsigs_mg_transformed.min() * 10) / 10
vmax = np.floor(varsigs_mg_transformed.max() * 10) / 10
# cbar_varsig.ax.get_xaxis().set_ticks([vmin, vmax])
# cbar_varsig.ax.get_xaxis().labelpad = 5
# cbar_varsig.set_label('$p(c_1)$')
# ax_varsig.set_title('$\log(10\sigma_\\bot(\mathcal{L})+1)$')
ax_varsig.xaxis.set_major_formatter(FormatStrFormatter('%.1f'))
ax_varsig.yaxis.set_major_formatter(FormatStrFormatter('%.1f'))
fig_varsig.set_size_inches(mm2in * single_width / 2 * scale_factor, mm2in * single_height / 2 * scale_factor, forward=True)
fig_varsig.subplots_adjust(
    top=0.90,
    bottom=0.15,
    left=0.15,
    right=0.97,
    hspace=0.05,
    wspace=0.05
)
# fig_varsig.tight_layout(pad=0.2)
# fig_varsig.tight_layout(pad=0.2)
# fig_varsig.tight_layout(pad=0.2)
fig_varsig.tight_layout(pad=0.1)
fig_varsig.tight_layout(pad=0.1)
fig_varsig.tight_layout(pad=0.1)

ax_iidood_varsig.set_xlabel('$d$', labelpad=-15)
ax_iidood_varsig.set_ylabel('$\log(10\sigma)$', labelpad=0)
range_y = max(varsigs_test_flat_transformed) - min(varsigs_test_flat_transformed)
dmaxOoD = max(d_OoD_flat.numpy())
vmax = np.floor(varsigs_mg_transformed.max() * 10) / 10
if dataset == "two_moons":
    ax_iidood_varsig.set_xticks([0.0, 0.2, *np.arange(np.round(dmaxOoD), 1.0), np.round(dmaxOoD)])
if dataset == "stripes":
    ax_iidood_varsig.set_xticks([0.0, 0.5, *np.arange(np.round(dmaxOoD), 1.0), np.round(dmaxOoD)])
ax_iidood_varsig.set_yticks([0.0,
                             # np.round(min(varsigs_test_flat_transformed), decimals=1),
                             np.round(thresh_varsig_test, decimals=1),
                             # np.round(max(varsigs_test_flat_transformed), decimals=1),
                             vmax])
ax_iidood_varsig.tick_params(axis='x', labelrotation=0)
ax_iidood_varsig.tick_params(axis='y', labelrotation=0)
for tick in ax_iidood_varsig.xaxis.get_major_ticks()[1::2]:
    tick.set_pad(15)
# ax_iidood_varsig.legend(handles=[ood_scatter, pdf_density],
#                         labels=['OoD samples', '$p_{test}(\log(10\sigma_\\bot(\mathcal{L})+1))$'],
#                         loc='upper left')
ax_iidood_varsig.xaxis.set_major_formatter(FormatStrFormatter('%.1f'))
ax_iidood_varsig.yaxis.set_major_formatter(FormatStrFormatter('%.1f'))
# ax_iidood_varsig.set_title('Uncertainty over Distance')
fig_iidood_varsig.set_size_inches(mm2in * single_width / 2 * scale_factor, mm2in * single_height * scale_factor, forward=True)
fig_iidood_varsig.set_size_inches(mm2in * single_width / 2 * scale_factor, mm2in * single_height * scale_factor, forward=True)
fig_iidood_varsig.tight_layout(pad=0.0)
fig_iidood_varsig.tight_layout(pad=0.0)
fig_iidood_varsig.tight_layout(pad=0.0)
fig_iidood_varsig.subplots_adjust(
    top=0.99,
    bottom=0.17,
    left=0.25,
    right=0.96,
    hspace=0.05,
    wspace=0.05
)


# ax_not_caught_varsig.set_xlabel('$x_1$', labelpad=-10)
# ax_not_caught_varsig.set_ylabel('$x_2$', labelpad=-17)
# ax_not_caught_varsig.set_aspect('equal', adjustable='box')
if dataset == "stripes":
    ax_not_caught_varsig.set_xlim([x0_min - 3.0, x0_max + 3.0])
    ax_not_caught_varsig.set_ylim([x1_min - 3.0, x1_max + 3.0])
if dataset == "two_moons":
    ax_not_caught_varsig.set_xlim([x0_min - 1.0, x0_max + 1.0])
    ax_not_caught_varsig.set_ylim([x1_min - 1.0, x1_max + 1.0])
# ax_not_caught_varsig.set_xticks([x0_min, x0_max])
# ax_not_caught_varsig.set_xticklabels([str(x0_min), str(x0_max)])
# ax_not_caught_varsig.set_yticks([x1_min, x1_max])
# ax_not_caught_varsig.set_yticklabels([str(x1_min), str(x1_max)])
ax_not_caught_varsig.set_xticks([])
ax_not_caught_varsig.set_xticklabels([])
ax_not_caught_varsig.set_yticks([])
ax_not_caught_varsig.set_yticklabels([])
vmin = np.ceil(varsigs_mg_transformed.min() * 10) / 10
vmax = np.floor(varsigs_mg_transformed.max() * 10) / 10
# cbar_not_caught_varsig.ax.get_xaxis().set_ticks([vmin, vmax])
# ax_not_caught_varsig.legend(handles=[not_caught_scatter],
#                         labels=["OoD samples"])
# ax_not_caught_varsig.set_title('$\log(10\sigma_\\bot(\mathcal{L})+1)$')
ax_not_caught_varsig.xaxis.set_major_formatter(FormatStrFormatter('%.1f'))
ax_not_caught_varsig.yaxis.set_major_formatter(FormatStrFormatter('%.1f'))
fig_not_caught_varsig.set_size_inches(mm2in * single_width / 2 * scale_factor, mm2in * single_height / 2 * scale_factor, forward=True)
fig_not_caught_varsig.subplots_adjust(
    top=0.90,
    bottom=0.15,
    left=0.15,
    right=0.97,
    hspace=0.05,
    wspace=0.05
)
fig_not_caught_varsig.tight_layout(pad=0.1)
fig_not_caught_varsig.tight_layout(pad=0.1)
fig_not_caught_varsig.tight_layout(pad=0.1)

# plot a simple colorbar


plt.show(block=False)

if dataset == "stripes":
    savepath = Path("C:\\Users\\LindenmeyerA\\Documents\\Inadequacy2\\Pictures\\SmallScale\\ENN\\stripes")
    plt.figure(fig_pred.number)
    fig_pred.savefig(savepath.joinpath('enn_stripes_pred.pgf'), backend='pgf', dpi=100)
    plt.figure(fig_varsig.number)
    fig_varsig.savefig(savepath.joinpath('enn_stripes_varsig.pgf'), backend='pgf', dpi=100)

    savepath = Path("C:\\Users\\LindenmeyerA\\Documents\\Inadequacy2\\Pictures\\SmallScale\\ENN\\stripesOoD")
    plt.figure(fig_iidood_varsig.number)
    fig_iidood_varsig.savefig(savepath.joinpath('enn_stripes_UvD.pgf'), backend='pgf', dpi=100)
    plt.figure(fig_not_caught_varsig.number)
    fig_not_caught_varsig.savefig(savepath.joinpath('enn_stripes_NC.pgf'), backend='pgf', dpi=100)


if dataset == "two_moons":
    savepath = Path("C:\\Users\\LindenmeyerA\\Documents\\Inadequacy2\\Pictures\\SmallScale\\ENN\\twomoons")
    plt.figure(fig_pred.number)
    fig_pred.savefig(savepath.joinpath('enn_two_moons_pred.pgf'), backend='pgf', dpi=100)
    plt.figure(fig_varsig.number)
    fig_varsig.savefig(savepath.joinpath('enn_two_moons_varsig.pgf'), backend='pgf', dpi=100)

    savepath = Path("C:\\Users\\LindenmeyerA\\Documents\\Inadequacy2\\Pictures\\SmallScale\\ENN\\twomoonsOoD")
    plt.figure(fig_iidood_varsig.number)
    fig_iidood_varsig.savefig(savepath.joinpath('enn_two_moons_UvD.pgf'), backend='pgf', dpi=100)
    plt.figure(fig_not_caught_varsig.number)
    fig_not_caught_varsig.savefig(savepath.joinpath('enn_two_moons_NC.pgf'), backend='pgf', dpi=100)



