# proof 1 sin high dataavailability
import matplotlib

matplotlib.use('TkAgg')
import matplotlib.pyplot as plt
import itertools
from datastuff.data import generate_data_classification_twomoons, \
    generate_data_classification_twomoons_OoD, \
    generate_data_classification_stripes_v2, \
    generate_data_classification_stripes_v2_OoD
import torch
import torch.nn as nn
import torch.nn.functional as ff
from torch.optim.adam import Adam
import numpy as np
from helpers.batch_indexer import BatchIndexerShuffle, BatchIndexerSeq
from helpers.aucroc import aucroc_multiclass
from SmallScale.sngp.spectralnormalization import spectral_norm
from SmallScale.sngp.gplastlay import SNGP_Classification_simple
from helpers.calibplot import sliding_window_calib_curve
from copy import deepcopy
from scipy.stats.kde import gaussian_kde
from pathlib import Path
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
from matplotlib.ticker import FormatStrFormatter
from sklearn.metrics import precision_recall_curve, auc, roc_auc_score


device = torch.device('cuda')


class Net(nn.Module):
    def __init__(self,
                 in_size: int,
                 num_layers: int,
                 hidden_size: int):
        super(Net, self).__init__()

        assert num_layers >= 1
        self.num_layers = num_layers

        # first default layer
        self.linlay_0 = nn.Linear(in_size, hidden_size)

        # main model body
        self.modelbody = []
        for i in range(num_layers):
            self.modelbody.append(spectral_norm(nn.Linear(hidden_size, hidden_size), norm_bound=0.9))
            # self.modelbody.append(nn.Linear(hidden_size, hidden_size))
        self.modelbody = nn.ModuleList(self.modelbody)

        # out layer
        self.gp_lay = SNGP_Classification_simple(in_size=hidden_size, out_size=2, random_feature_size=1024, normalize_input=False, gp_kernel_scale=5.0, sigma=1.5)

    def forward(self, x):
        x = self.linlay_0(x)

        for i in range(0, self.num_layers):
            linlay = self.modelbody[i]
            x = x + linlay(torch.relu(x))
            # x = linlay(torch.relu(x))

        x = self.gp_lay(x)
        # watch out the x will be different depending on the mode the gplayer is in
        return x


################## DATA ############################
# data
dataset = "two_moons"
if dataset == "stripes":
    x, y, x_eval, y_eval, x_test, y_test = generate_data_classification_stripes_v2()
if dataset == "two_moons":
    x, y, x_eval, y_eval, x_test, y_test = generate_data_classification_twomoons()

# OoD data
if dataset == "stripes":
    x_OoD, d_OoD_flat = generate_data_classification_stripes_v2_OoD()
if dataset == "two_moons":
    x_OoD, d_OoD_flat = generate_data_classification_twomoons_OoD()

# grid data
grid_shape_0 = 100
grid_shape_1 = 80
x0_min, x0_max, x1_min, x1_max = x[:, 0].min().item(), x[:, 0].max().item(), x[:, 1].min().item(), x[:, 1].max().item()
if dataset == "stripes":
    x0_test_vector = torch.linspace(x0_min - 3.0, x0_max + 3.0, grid_shape_0)
    x1_test_vector = torch.linspace(x1_min - 3.0, x1_max + 3.0, grid_shape_1)
if dataset == "two_moons":
    x0_test_vector = torch.linspace(x0_min - 1.0, x0_max + 1.0, grid_shape_0)
    x1_test_vector = torch.linspace(x1_min - 1.0, x1_max + 1.0, grid_shape_1)
x0_test_mg, x1_test_mg = torch.meshgrid(x0_test_vector, x1_test_vector)
x0_test_mg_flat = x0_test_mg.reshape(-1)
x1_test_mg_flat = x1_test_mg.reshape(-1)
x_test_mg_flat = torch.stack([x0_test_mg_flat, x1_test_mg_flat], dim=1)

################## TRAIN ############################

loss = ff.cross_entropy

x = x.to(device)
y = y.to(device)
x_eval = x_eval.to(device)
y_eval = y_eval.to(device)
x_test = x_test.to(device)
y_test = y_test.to(device)

num_layers = [1, 2, 3]
num_neurons = [100, 120, 150]
net_configurations = list(itertools.product(num_layers, num_neurons))
net_configurations = [item for item in net_configurations for _ in range(1)]

num_nets = len(net_configurations)

max_patience = 10
max_epochs = 8000

# for saving and later evaluating MI
net_predictions_pred_mg_flat_mean = []
net_predictions_cov_mg_flat_mean = []
net_predictions_pred_test_flat = []
net_predictions_cov_test_flat = []
net_predictions_pred_OoD_flat = []
net_predictions_cov_OoD_flat = []

# move to cuda
for i in range(num_nets):
    current_best_loss = torch.tensor(9999.0)
    current_best_model = None
    bidexer_train = BatchIndexerShuffle(len(x), None, 600, device, True)
    bidexer_eval = BatchIndexerSeq(len(x_eval), None, 100, device, False)
    net_config = net_configurations[i]
    net = Net(2, net_config[0], net_config[1])

    current_patience = 0
    interval = 5

    net = net.to(device)

    optim = Adam(net.parameters(), lr=0.001, weight_decay=0.0005)

    net.train()

    for epoch in range(max_epochs):
        epoch_loss = 0
        for b_idxs in bidexer_train.generate_b_idx():
            b_inputs = x[b_idxs, :]
            b_targets = y[b_idxs]
            optim.zero_grad()
            pred = net(b_inputs)
            _loss = loss(pred, b_targets, reduction='mean', label_smoothing=0.001)
            epoch_loss += _loss.item()
            _loss.backward()
            optim.step()
            epoch_loss += _loss.item()
        if epoch % interval == 0:
            # do eval
            eval_loss = 0
            eval_preds = []
            eval_trgs = []
            with torch.no_grad():
                for b_idxs in bidexer_eval.generate_b_idx():
                    b_inputs = x_eval[b_idxs, :]
                    b_targets = y_eval[b_idxs]
                    pred = net(b_inputs)
                    _loss = loss(pred, b_targets, reduction='sum', label_smoothing=0.001)
                    eval_loss += _loss.item()
                    eval_preds.append(pred)
                    eval_trgs.append(b_targets)
            eval_preds = torch.concat(eval_preds, dim=0)
            eval_trgs = torch.concat(eval_trgs, dim=0)
            avg_score, _, _ = aucroc_multiclass(nn.functional.softmax(eval_preds, dim=-1).detach().cpu().numpy(),
                                                eval_trgs.view(-1).detach().cpu().numpy(),
                                                averaging='weighted')

            print('---------------------')
            print('EPOCH {}/{}'.format(epoch, max_epochs))
            print('EPOCH_LOSS: {}'.format(epoch_loss))
            print('EVAL_LOSS: {}'.format(eval_loss))
            print('AUCROC: {}'.format(avg_score))
            if eval_loss < current_best_loss:
                current_patience = 0
                current_best_loss = eval_loss
                current_best_model = net.state_dict()
            else:
                current_patience += 1
            print('PATIENCE {}/{}'.format(current_patience, max_patience))
            if current_patience >= max_patience:
                break

    ################## DATA COLLECTION ############################

    net.eval()
    net.gp_lay.finalize()
    # run through the dataset to collect the precision matrix
    collection_cycles = 30
    with torch.no_grad():
        for cci in range(collection_cycles):
            print("collecting precision {}/{}".format(cci, collection_cycles))
            for b_idxs in bidexer_train.generate_b_idx():
                b_inputs = x[b_idxs, :]
                net(b_inputs)

    # generate the covariance matrix
    net.gp_lay.gen_laplace_approx()
    net.eval()

    # get normal pred for entire grid
    with torch.no_grad():
        x_test_mg_flat = x_test_mg_flat.to(device)
        pred, cov = net(x_test_mg_flat)
        pred = pred.cpu()
        cov = cov.cpu()
    net_predictions_pred_mg_flat_mean.append(pred)
    net_predictions_cov_mg_flat_mean.append(cov)

    # get predictions on test points defined trough the dataset
    with torch.no_grad():
        x_test = x_test.to(device)
        pred, cov = net(x_test)
        pred = pred.cpu()
        cov = cov.cpu()
    net_predictions_pred_test_flat.append(pred)
    net_predictions_cov_test_flat.append(cov)

    # get predictions on OoD data
    with torch.no_grad():
        x_OoD = x_OoD.to(device)
        pred, cov = net(x_OoD)
        pred = pred.cpu()
        cov = cov.cpu()
    net_predictions_pred_OoD_flat.append(pred)
    net_predictions_cov_OoD_flat.append(cov)

################## POSTPROC ############################

# for the grid
net_predictions_pred_mg_flat_mean = torch.softmax(torch.mean(torch.stack(net_predictions_pred_mg_flat_mean, dim=1), dim=1), dim=-1)[:, 0]
net_predictions_cov_mg_flat_mean = torch.mean(torch.stack(net_predictions_cov_mg_flat_mean, dim=1), dim=1)

# for the test set
net_predictions_pred_test_flat_mean = torch.softmax(torch.mean(torch.stack(net_predictions_pred_test_flat, dim=1), dim=1), dim=1)
net_predictions_pred_test_flat_mean_0 = net_predictions_pred_test_flat_mean[:, 0]
net_predictions_cov_test_flat_mean = torch.mean(torch.stack(net_predictions_cov_test_flat, dim=1), dim=1)

# get the aucroc and aucpr measures
precision, recall, _ = precision_recall_curve(y_test.cpu().numpy(), net_predictions_pred_test_flat_mean[:, 1])
aucpr = auc(recall, precision)
aucroc = roc_auc_score(y_test.cpu().numpy(), net_predictions_pred_test_flat_mean[:, 1])
print("AUCROC: {}".format(aucroc))
print("AUCPR: {}".format(aucpr))

# for the OoD set
net_predictions_cov_OoD_flat_mean = torch.mean(torch.stack(net_predictions_cov_OoD_flat, dim=1), dim=1)
net_predictions_pred_OoD_flat_mean = torch.softmax(torch.mean(torch.stack(net_predictions_pred_OoD_flat, dim=1), dim=1), dim=1)
net_predictions_pred_OoD_flat_mean_0 = net_predictions_pred_OoD_flat_mean[:, 0]
################## MEASURES ############################

# calculate cov for mg
cov_mg = net_predictions_cov_mg_flat_mean.reshape(grid_shape_0, grid_shape_1).numpy()

# calculate cov for OoD
cov_OoD_flat = net_predictions_cov_OoD_flat_mean.numpy()

# calculate cov for test
cov_test_flat = net_predictions_cov_test_flat_mean.numpy()

# calculate calibration
true_frac, pred_frac = sliding_window_calib_curve(net_predictions_pred_test_flat_mean.cpu().numpy(), y_test.cpu().numpy(), bin_size=250)

################## PLOT ############################

x0_test_mg_flat = x0_test_mg_flat.numpy()
x1_test_mg_flat = x1_test_mg_flat.numpy()

pred_mg = net_predictions_pred_mg_flat_mean.reshape(grid_shape_0, grid_shape_1).numpy()

fig_pred, ax_pred= plt.subplots()

# plot the actual prediction
prediction_image = ax_pred.imshow(np.transpose(pred_mg), vmin=pred_mg.min(), vmax=pred_mg.max(), origin='lower', interpolation='bilinear',
                                extent=[x0_test_mg_flat.min(), x0_test_mg_flat.max(),
                                        x1_test_mg_flat.min(), x1_test_mg_flat.max()])
# cbar_pred = plt.colorbar(prediction_image, ax=ax_pred, orientation='horizontal', shrink=0.75)
ax_pred.scatter(x[:, 0].view(-1).cpu().numpy(), x[:, 1].view(-1).cpu().numpy(), s=1, c="w", rasterized=True)

fig_cov, ax_cov = plt.subplots()

# run through log to show minute details in the lower ranges
cov_mg_transformed = np.log(cov_mg * 10 + 1)
uncertainty_image = ax_cov.imshow(np.transpose(cov_mg_transformed), vmin=cov_mg_transformed.min(), vmax=cov_mg_transformed.max(), origin='lower', interpolation='bilinear',
                                  extent=[x0_test_mg_flat.min(), x0_test_mg_flat.max(),
                                          x1_test_mg_flat.min(), x1_test_mg_flat.max()])
# cbar_cov = fig_cov.colorbar(uncertainty_image, ax=ax_cov, orientation='horizontal', shrink=0.75)
ax_cov.scatter(x[:, 0].view(-1).cpu().numpy(), x[:, 1].view(-1).cpu().numpy(), s=1, c="w", rasterized=True)
plt.show(block=False)

# # calibration plot
# fig = plt.figure()
# ax = fig.subplots(1, 1)

# # plot the calibration
# ax.plot(pred_frac, true_frac)
# ax.plot(np.array([0, 1]), np.array([0, 1]))
# ax.set_xlabel("pred fraction")
# ax.set_ylabel("true fraction")
# plt.show(block=False)

# plot distance to uncertainty
fig_iidood_cov, ax_iidood_cov= plt.subplots()

cov_OoD_flat_transformed = np.log(cov_OoD_flat * 10 + 1)
cov_test_flat_transformed = np.log(cov_test_flat * 10 + 1)
ood_scatter = ax_iidood_cov.scatter(d_OoD_flat.cpu().numpy(), cov_OoD_flat_transformed, s=1, rasterized=True)
if dataset == "two_moons":
    pdf_test = gaussian_kde(cov_test_flat_transformed, bw_method=0.05)
if dataset == "stripes":
    pdf_test = gaussian_kde(cov_test_flat_transformed, bw_method=0.015)
range_y = max(cov_test_flat_transformed) - min(cov_test_flat_transformed)
pdf_test_y = np.linspace(min(cov_test_flat_transformed)-range_y/4, max(cov_test_flat_transformed)+range_y/4, 1200)
pdf_test_x = pdf_test(pdf_test_y)
pdf_test_x = - pdf_test_x / (4*max(pdf_test_x))
# clamp to 0 below 0 or delete
pdf_test_x = pdf_test_x[pdf_test_y >= 0.0]
pdf_test_y = pdf_test_y[pdf_test_y >= 0.0]
ax_iidood_cov.plot(pdf_test_x, pdf_test_y, color="tab:orange")
pdf_density = ax_iidood_cov.fill_betweenx(pdf_test_y, np.zeros_like(pdf_test_x), pdf_test_x, color="tab:orange", alpha=0.5)
cut_percentile = np.percentile(cov_test_flat_transformed, 90)
not_caught_shuffled = cov_OoD_flat_transformed < cut_percentile
print("not caught percent: {}".format(np.sum(not_caught_shuffled)/len(not_caught_shuffled)))
ax_iidood_cov.plot([min(pdf_test_x), 1], [cut_percentile, cut_percentile], color="black")
ax_iidood_cov.set_xlabel("distance")
ax_iidood_cov.set_ylabel("uncertainty")
plt.show(block=False)

thresh_cov_test = np.quantile(cov_test_flat_transformed, 0.9)
not_caught_cov = cov_OoD_flat_transformed.flatten() < thresh_cov_test
rejected_cov = cov_test_flat_transformed > thresh_cov_test
fig_not_caught_cov, ax_not_caught_cov= plt.subplots()
cov_mg_transformed = np.log(cov_mg * 10 + 1)
uncertainty_image = ax_not_caught_cov.imshow(np.transpose(cov_mg_transformed), vmin=cov_mg_transformed.min(), vmax=cov_mg_transformed.max(), origin='lower', interpolation='bilinear',
                                             extent=[x0_test_mg_flat.min(), x0_test_mg_flat.max(),
                                                     x1_test_mg_flat.min(), x1_test_mg_flat.max()])
# cbar_not_caught_cov = fig_not_caught_cov.colorbar(uncertainty_image, ax=ax_not_caught_cov, orientation='horizontal', shrink=0.75)
ax_not_caught_cov.scatter(x[:, 0].view(-1).cpu().numpy(), x[:, 1].view(-1).cpu().numpy(), s=1, c="w", marker='.', rasterized=True)
not_caught_scatter = ax_not_caught_cov.scatter(x_OoD[not_caught_cov, 0].view(-1).cpu().numpy(), x_OoD[not_caught_cov, 1].view(-1).cpu().numpy(), s=12, c="r", marker='x', rasterized=True)

plt.show(block=False)
############## FORMATTING
cm2in = 1.0 / 2.54
mm2in = 1.0 / 25.4
single_height = 74
single_width = 74
double_width = 140
third_width = double_width/3
third_height = single_height * 0.66
scale_factor = 1
plt.rcParams['pgf.texsystem'] = 'xelatex'


x0_min, x0_max, x1_min, x1_max = np.round(x0_min, 1), np.round(x0_max, 1), np.round(x1_min, 1), np.round(x1_max, 1)
# ax_pred.set_xlabel('$x_1$', labelpad=-10)
# ax_pred.set_ylabel('$x_2$', labelpad=-17)
# ax_pred.set_aspect('equal', adjustable='box')
if dataset == "stripes":
    ax_pred.set_xlim([x0_min - 3.0, x0_max + 3.0])
    ax_pred.set_ylim([x1_min - 3.0, x1_max + 3.0])
if dataset == "two_moons":
    ax_pred.set_xlim([x0_min - 1.0, x0_max + 1.0])
    ax_pred.set_ylim([x1_min - 1.0, x1_max + 1.0])
# ax_pred.set_xticks([x0_min, x0_max])
# ax_pred.set_xticklabels([str(x0_min), str(x0_max)])
# ax_pred.set_yticks([x1_min, x1_max])
# ax_pred.set_yticklabels([str(x1_min), str(x1_max)])
ax_pred.set_xticks([])
ax_pred.set_xticklabels([])
ax_pred.set_yticks([])
ax_pred.set_yticklabels([])
vmin = np.ceil(pred_mg.min() * 10) / 10
vmax = np.floor(pred_mg.max() * 10) / 10
# cbar_pred.ax.get_xaxis().set_ticks([vmin, vmax])
# cbar_pred.ax.get_xaxis().labelpad = 5
# cbar_pred.set_label('$p(c_1)$')
# ax_pred.set_title('$p(c_0)$')
ax_pred.xaxis.set_major_formatter(FormatStrFormatter('%.1f'))
ax_pred.yaxis.set_major_formatter(FormatStrFormatter('%.1f'))
fig_pred.set_size_inches(mm2in * single_width / 2 * scale_factor, mm2in * single_height / 2 * scale_factor, forward=True)
fig_pred.subplots_adjust(
    top=0.90,
    bottom=0.15,
    left=0.15,
    right=0.97,
    hspace=0.05,
    wspace=0.05
)
fig_pred.tight_layout(pad=0.1)
fig_pred.tight_layout(pad=0.1)
fig_pred.tight_layout(pad=0.1)

# ax_cov.set_xlabel('$x_1$', labelpad=-10)
# ax_cov.set_ylabel('$x_2$', labelpad=-17)
# ax_cov.set_aspect('equal', adjustable='box')
if dataset == "stripes":
    ax_cov.set_xlim([x0_min - 3.0, x0_max + 3.0])
    ax_cov.set_ylim([x1_min - 3.0, x1_max + 3.0])
if dataset == "two_moons":
    ax_cov.set_xlim([x0_min - 1.0, x0_max + 1.0])
    ax_cov.set_ylim([x1_min - 1.0, x1_max + 1.0])
# ax_cov.set_xticks([x0_min, x0_max])
# ax_cov.set_xticklabels([str(x0_min), str(x0_max)])
# ax_cov.set_yticks([x1_min, x1_max])
# ax_cov.set_yticklabels([str(x1_min), str(x1_max)])
ax_cov.set_xticks([])
ax_cov.set_xticklabels([])
ax_cov.set_yticks([])
ax_cov.set_yticklabels([])
vmin = np.ceil(cov_mg_transformed.min() * 10) / 10
vmax = np.floor(cov_mg_transformed.max() * 10) / 10
# cbar_cov.ax.get_xaxis().set_ticks([vmin, vmax])
# cbar_cov.ax.get_xaxis().labelpad = 5
# cbar_cov.set_label('$p(c_1)$')
# ax_cov.set_title('$\log(10\sigma+1)$')
ax_cov.xaxis.set_major_formatter(FormatStrFormatter('%.1f'))
ax_cov.yaxis.set_major_formatter(FormatStrFormatter('%.1f'))
fig_cov.set_size_inches(mm2in * single_width / 2 * scale_factor, mm2in * single_height / 2 * scale_factor, forward=True)
fig_cov.subplots_adjust(
    top=0.90,
    bottom=0.15,
    left=0.15,
    right=0.97,
    hspace=0.05,
    wspace=0.05
)
fig_cov.tight_layout(pad=0.1)
fig_cov.tight_layout(pad=0.1)
fig_cov.tight_layout(pad=0.1)

ax_iidood_cov.set_xlabel('$d$', labelpad=-15)
ax_iidood_cov.set_ylabel('$\log(10\sigma)$', labelpad=0)
range_y = max(cov_test_flat_transformed) - min(cov_test_flat_transformed)
dmaxOoD = max(d_OoD_flat.numpy())
vmax = np.floor(cov_mg_transformed.max() * 10) / 10
if dataset == "two_moons":
    ax_iidood_cov.set_xticks([0.0, 0.2, *np.arange(np.round(dmaxOoD), 1.0), np.round(dmaxOoD)])
if dataset == "stripes":
    ax_iidood_cov.set_xticks([0.0, 0.5, *np.arange(np.round(dmaxOoD), 1.0), np.round(dmaxOoD)])
# ax_iidood_cov.set_yticks([0.0,
#                           np.round(min(cov_test_flat_transformed), decimals=1),
#                           thresh_cov_test,
#                           np.round(max(cov_test_flat_transformed), decimals=1),
#                           vmax])
ax_iidood_cov.set_yticks([0.0,
                          np.round(vmax, decimals=1)])
ax_iidood_cov.tick_params(axis='x', labelrotation=0)
ax_iidood_cov.tick_params(axis='y', labelrotation=0)
for tick in ax_iidood_cov.xaxis.get_major_ticks()[1::2]:
    tick.set_pad(15)
# ax_iidood_cov.legend(handles=[ood_scatter, pdf_density],
#                      labels=['OoD samples', '$p_{test}(\log(10\sigma+1))$'],
#                      loc='upper left')
inset_ax = inset_axes(ax_iidood_cov, width="40%", height="80%", loc='lower right')
inset_ax.plot(pdf_test_x, pdf_test_y, color="tab:orange")
inset_ax.fill_betweenx(pdf_test_y, np.zeros_like(pdf_test_x), pdf_test_x, color="tab:orange", alpha=0.5)
inset_ax.plot([min(pdf_test_x), 1.0], [cut_percentile, cut_percentile], color="black")
inset_ax.scatter(d_OoD_flat.cpu().numpy(), cov_OoD_flat_transformed, s=1, rasterized=True)
inset_ax.set_xlim([min(pdf_test_x) - 0.1, 0.1])
# inset_ax.set_ylim([min(pdf_test_y) + range_y/8, max(pdf_test_y) - range_y*1.1])
if dataset == "stripes":
    inset_ax.set_ylim([min(pdf_test_y) - range_y * 0.004, max(pdf_test_y) - range_y * 1.205])
if dataset == "two_moons":
    inset_ax.set_ylim([min(pdf_test_y) - range_y * 0.004, max(pdf_test_y) - range_y * 1.15])
inset_ax.set_xticks([])
inset_ax.set_yticks([0.0,
                     thresh_cov_test])
inset_ax.set_yticklabels(["0.0",
                          "{}e-3".format(np.round(thresh_cov_test*1000, decimals=1))])
inset_ax.tick_params(axis='x', labelrotation=0)
inset_ax.tick_params(axis='y', labelrotation=0)
for label in inset_ax.get_yticklabels():
    label.set_bbox(dict(facecolor='white', edgecolor='none', alpha=0.85, boxstyle='round,pad=0.1'))

ax_iidood_cov.xaxis.set_major_formatter(FormatStrFormatter('%.1f'))
ax_iidood_cov.yaxis.set_major_formatter(FormatStrFormatter('%.1f'))
# ax_iidood_cov.set_title('Uncertainty over Distance')
fig_iidood_cov.set_size_inches(mm2in * single_width / 2 * scale_factor, mm2in * single_height * scale_factor, forward=True)
fig_iidood_cov.set_size_inches(mm2in * single_width / 2 * scale_factor, mm2in * single_height * scale_factor, forward=True)
fig_iidood_cov.tight_layout(pad=0.0)
fig_iidood_cov.tight_layout(pad=0.0)
fig_iidood_cov.tight_layout(pad=0.0)
fig_iidood_cov.subplots_adjust(
    top=0.99,
    bottom=0.17,
    left=0.25,
    right=0.97,
    hspace=0.05,
    wspace=0.05
)

# ax_not_caught_cov.set_xlabel('$x_1$', labelpad=-10)
# ax_not_caught_cov.set_ylabel('$x_2$', labelpad=-17)
# ax_not_caught_cov.set_aspect('equal', adjustable='box')
if dataset == "stripes":
    ax_not_caught_cov.set_xlim([x0_min - 3.0, x0_max + 3.0])
    ax_not_caught_cov.set_ylim([x1_min - 3.0, x1_max + 3.0])
if dataset == "two_moons":
    ax_not_caught_cov.set_xlim([x0_min - 1.0, x0_max + 1.0])
    ax_not_caught_cov.set_ylim([x1_min - 1.0, x1_max + 1.0])
# ax_not_caught_cov.set_xticks([x0_min, x0_max])
# ax_not_caught_cov.set_xticklabels([str(x0_min), str(x0_max)])
# ax_not_caught_cov.set_yticks([x1_min, x1_max])
# ax_not_caught_cov.set_yticklabels([str(x1_min), str(x1_max)])
ax_not_caught_cov.set_xticks([])
ax_not_caught_cov.set_xticklabels([])
ax_not_caught_cov.set_yticks([])
ax_not_caught_cov.set_yticklabels([])
vmin = np.ceil(cov_mg_transformed.min() * 10) / 10
vmax = np.floor(cov_mg_transformed.max() * 10) / 10
# cbar_not_caught_cov.ax.get_xaxis().set_ticks([vmin, vmax])
# ax_not_caught_cov.legend(handles=[not_caught_scatter],
#                         labels=["OoD samples"])
# ax_not_caught_cov.set_title('$\log(10\sigma+1)$')
ax_not_caught_cov.xaxis.set_major_formatter(FormatStrFormatter('%.1f'))
ax_not_caught_cov.yaxis.set_major_formatter(FormatStrFormatter('%.1f'))
fig_not_caught_cov.set_size_inches(mm2in * single_width / 2 * scale_factor, mm2in * single_height / 2 * scale_factor, forward=True)
fig_not_caught_cov.subplots_adjust(
    top=0.90,
    bottom=0.15,
    left=0.15,
    right=0.96,
    hspace=0.05,
    wspace=0.05
)
fig_not_caught_cov.tight_layout(pad=0.1)
fig_not_caught_cov.tight_layout(pad=0.1)
fig_not_caught_cov.tight_layout(pad=0.1)

plt.show(block=False)

if dataset == "two_moons":
    savepath = Path("C:\\Users\\LindenmeyerA\\Documents\\Inadequacy2\\Pictures\\SmallScale\\SNGP\\twomoons")
    plt.figure(fig_pred.number)
    fig_pred.savefig(savepath.joinpath('sngp_two_moons_pred.pgf'), backend='pgf', dpi=100)
    plt.figure(fig_cov.number)
    fig_cov.savefig(savepath.joinpath('sngp_two_moons_cov.pgf'), backend='pgf', dpi=100)

    savepath = Path("C:\\Users\\LindenmeyerA\\Documents\\Inadequacy2\\Pictures\\SmallScale\\SNGP\\twomoonsOoD")
    plt.figure(fig_iidood_cov.number)
    fig_iidood_cov.savefig(savepath.joinpath('sngp_two_moons_UvD.pgf'), backend='pgf', dpi=100)
    plt.figure(fig_not_caught_cov.number)
    fig_not_caught_cov.savefig(savepath.joinpath('sngp_two_moons_NC.pgf'), backend='pgf', dpi=100)

if dataset == "stripes":
    savepath = Path("C:\\Users\\LindenmeyerA\\Documents\\Inadequacy2\\Pictures\\SmallScale\\SNGP\\stripes")
    plt.figure(fig_pred.number)
    fig_pred.savefig(savepath.joinpath('sngp_stripes_pred.pgf'), backend='pgf', dpi=100)
    plt.figure(fig_cov.number)
    fig_cov.savefig(savepath.joinpath('sngp_stripes_cov.pgf'), backend='pgf', dpi=100)

    savepath = Path("C:\\Users\\LindenmeyerA\\Documents\\Inadequacy2\\Pictures\\SmallScale\\SNGP\\stripesOoD")
    plt.figure(fig_iidood_cov.number)
    fig_iidood_cov.savefig(savepath.joinpath('sngp_stripes_UvD.pgf'), backend='pgf', dpi=100)
    plt.figure(fig_not_caught_cov.number)
    fig_not_caught_cov.savefig(savepath.joinpath('sngp_stripes_NC.pgf'), backend='pgf', dpi=100)

