import torch
import torch.utils.data as tdatautils
import numpy as np
from sklearn import cluster, datasets
from scipy.spatial import KDTree
from typing import Literal

def generate_data_classification_stripes_v2(return_raw=False):
    #5d-s-9s-l-9d-m-1s-s-5s-l-1d
    # 5 = 50/50
    # 9 = 90/10
    # 1 = 10/90
    # Xd = dense
    # Xs = sparse
    # s = small gap = 1.0
    # m = med gap = 2.0
    # l = large gap = 3.0
    num_datapoints_init = 10000

    xy = torch.rand(num_datapoints_init, 2, generator=torch.Generator().manual_seed(69)) * 16.0
    x = xy[:, 0]
    y = xy[:, 1]
    trg = torch.zeros(num_datapoints_init, dtype=torch.int64)

    # 5d
    filtr = (0.0 < x) & (x < 1.0)
    trg[filtr] = torch.round(torch.rand(trg[filtr].shape, generator=torch.Generator().manual_seed(69))).long()

    # s
    filtr = (1.0 < x) & (x < 2.0)
    trg = trg[~filtr]
    x = x[~filtr]
    y = y[~filtr]

    # 9s
    filtr = (2.0 < x) & (x < 3.0)
    rand_selection = torch.rand(filtr.shape, generator=torch.Generator().manual_seed(69)) < 0.8
    remaining = ~torch.logical_and(filtr, rand_selection)
    trg = trg[remaining]
    x = x[remaining]
    y = y[remaining]
    filtr = (2.0 < x) & (x < 3.0)
    trg[filtr] = (torch.rand(trg[filtr].shape, generator=torch.Generator().manual_seed(69)) < 0.9).long()

    # l
    filtr = (3.0 < x) & (x < 6.0)
    trg = trg[~filtr]
    x = x[~filtr]
    y = y[~filtr]

    # 9d
    filtr = (6.0 < x) & (x < 7.0)
    trg[filtr] = (torch.rand(trg[filtr].shape, generator=torch.Generator().manual_seed(69)) < 0.9).long()

    # m
    filtr = (7.0 < x) & (x < 9.0)
    trg = trg[~filtr]
    x = x[~filtr]
    y = y[~filtr]

    # 1s
    filtr = (9.0 < x) & (x < 10.0)
    rand_selection = torch.rand(filtr.shape, generator=torch.Generator().manual_seed(69)) < 0.8
    remaining = ~torch.logical_and(filtr, rand_selection)
    trg = trg[remaining]
    x = x[remaining]
    y = y[remaining]
    filtr = (9.0 < x) & (x < 10.0)
    trg[filtr] = (torch.rand(trg[filtr].shape, generator=torch.Generator().manual_seed(69)) < 0.1).long()

    # s
    filtr = (10.0 < x) & (x < 11.0)
    trg = trg[~filtr]
    x = x[~filtr]
    y = y[~filtr]

    # 5s
    filtr = (11.0 < x) & (x < 12.0)
    rand_selection = torch.rand(filtr.shape, generator=torch.Generator().manual_seed(69)) < 0.8
    remaining = ~torch.logical_and(filtr, rand_selection)
    trg = trg[remaining]
    x = x[remaining]
    y = y[remaining]
    filtr = (11.0 < x) & (x < 12.0)
    trg[filtr] = torch.round(torch.rand(trg[filtr].shape, generator=torch.Generator().manual_seed(69))).long()

    # l
    filtr = (12.0 < x) & (x < 15.0)
    trg = trg[~filtr]
    x = x[~filtr]
    y = y[~filtr]

    # 1d
    filtr = (15.0 < x) & (x < 16.0)
    trg[filtr] = (torch.rand(trg[filtr].shape, generator=torch.Generator().manual_seed(69)) < 0.1).long()

    num_datapoints = len(x)

    # import matplotlib
    # matplotlib.use('QtAgg')
    # import matplotlib.pyplot as plt
    # fig, ax = plt.subplots()
    # ax.scatter(x.numpy(), y.numpy(), c=trg)
    # plt.show(block=True)

    # split off eval data
    trainsplitlen = int(num_datapoints * 0.65)
    evalsplitlen = int((num_datapoints - trainsplitlen) / 2)
    testsplitlen = int(num_datapoints - trainsplitlen - evalsplitlen)
    split = tdatautils.random_split(range(num_datapoints), [trainsplitlen, evalsplitlen, testsplitlen], generator=torch.Generator().manual_seed(69))

    eval_data = torch.stack([x[split[1].indices], y[split[1].indices]], dim=-1)
    eval_trg = trg[split[1].indices]

    test_data = torch.stack([x[split[2].indices], y[split[2].indices]], dim=-1)
    test_trg = trg[split[2].indices]

    train_data = torch.stack([x[split[0].indices], y[split[0].indices]], dim=-1)
    train_trg = trg[split[0].indices]

    if return_raw:
        return x, y, trg
    return train_data, train_trg, eval_data, eval_trg, test_data, test_trg

def generate_data_classification_stripes_v2_OoD():
    # use train data
    X,z,_,_,_,_ = generate_data_classification_stripes_v2()

    x0_min, x0_max, x1_min, x1_max = X[:, 0].min().item() - 3.0, X[:, 0].max().item() + 3.0, X[:, 1].min().item() - 3.0, X[:, 1].max().item() + 3.0

    num_negative_points = 5000
    np.random.seed(69)
    X_neg = np.random.uniform(low=[x0_min, x1_min], high=[x0_max, x1_max], size=(num_negative_points, 2))

    tree = KDTree(X)
    distances, _ = tree.query(X_neg, k=1)
    threshold_distance = 0.5  # Adjust based on the noise level
    X_neg = X_neg[distances > threshold_distance]

    distances = calculate_distances(X, X_neg)

    X = X_neg
    z = distances

    X = torch.tensor(X, dtype=torch.float32)
    z = torch.tensor(z, dtype=torch.float32)

    x, y = X[:,0], X[:,1]

    # import matplotlib
    # matplotlib.use('QtAgg')
    # import matplotlib.pyplot as plt
    # from mpl_toolkits.mplot3d import Axes3D
    # fig1 = plt.figure()
    # ax1 = fig1.add_subplot(111, projection='3d')
    # ax1.scatter(x.numpy(), y.numpy(), z.numpy(), c=z.numpy(), cmap='viridis')
    # ax1.set_xlabel('x')
    # ax1.set_ylabel('y')
    # plt.show(block=True)

    return X, z

def generate_data_classification_twomoons(return_raw=False):
    X, z = datasets.make_moons(n_samples=2000, noise=0.05, random_state=69)

    X = torch.tensor(X, dtype=torch.float32)
    z = torch.tensor(z, dtype=torch.int64)

    # randomly flip some datapoints to simulate a little bit of noise
    torch.manual_seed(69)
    filtr = torch.rand(z.shape) < 0.08
    z[filtr] = torch.abs(z[filtr]-1)
    # z[filtr] = torch.randint(0, 2, z[filtr].shape)

    x, y = X[:,0], X[:,1]

    num_datapoints = x.shape[0]

    if len(x.shape) == 1:
        x = x.unsqueeze(-1)
    if len(y.shape) == 1:
        y = y.unsqueeze(-1)

    # split off eval data
    trainsplitlen = int(num_datapoints * 0.65)
    evalsplitlen = int((num_datapoints - trainsplitlen) / 2)
    testsplitlen = int(num_datapoints - trainsplitlen - evalsplitlen)
    split = tdatautils.random_split(range(num_datapoints), [trainsplitlen, evalsplitlen, testsplitlen], generator=torch.Generator().manual_seed(69))

    eval_data = torch.concat([x[split[1].indices], y[split[1].indices]], dim=1)
    eval_trg = z[split[1].indices]

    test_data = torch.concat([x[split[2].indices], y[split[2].indices]], dim=1)
    test_trg = z[split[2].indices]

    train_data = torch.concat([x[split[0].indices], y[split[0].indices]], dim=1)
    train_trg = z[split[0].indices]

    # import matplotlib
    # matplotlib.use('QtAgg')
    # import matplotlib.pyplot as plt
    # from mpl_toolkits.mplot3d import Axes3D
    # fig1 = plt.figure()
    # ax1 = fig1.add_subplot(111, projection='3d')
    # ax1.scatter(train_data[:, 0].numpy(), train_data[:, 1].numpy(), train_trg.numpy())
    # ax1.scatter(eval_data[:, 0].numpy(), eval_data[:, 1].numpy(), eval_trg.numpy())
    # ax1.set_xlabel('x')
    # ax1.set_ylabel('y')
    # plt.show(block=True)
    if return_raw:
        return X, z
    return train_data, train_trg, eval_data, eval_trg, test_data, test_trg

def generate_data_classification_twomoons_OoD():
    # use train data
    X,z,_,_,_,_ = generate_data_classification_twomoons()

    x0_min, x0_max, x1_min, x1_max = X[:, 0].min().item() - 1, X[:, 0].max().item() + 1, X[:, 1].min().item() - 1, X[:, 1].max().item() + 1

    num_negative_points = 10000
    np.random.seed(69)
    X_neg = np.random.uniform(low=[x0_min, x1_min], high=[x0_max, x1_max], size=(num_negative_points, 2))

    tree = KDTree(X)
    distances, _ = tree.query(X_neg, k=1)
    threshold_distance = 0.2  # Adjust based on the noise level
    X_neg = X_neg[distances > threshold_distance]

    distances = calculate_distances(X, X_neg)

    X = X_neg
    z = distances

    X = torch.tensor(X, dtype=torch.float32)
    z = torch.tensor(z, dtype=torch.float32)

    x, y = X[:,0], X[:,1]

    # import matplotlib
    # matplotlib.use('QtAgg')
    # import matplotlib.pyplot as plt
    # from mpl_toolkits.mplot3d import Axes3D
    # fig1 = plt.figure()
    # ax1 = fig1.add_subplot(111, projection='3d')
    # ax1.scatter(x.numpy(), y.numpy(), z.numpy(), c=z.numpy(), cmap='viridis')
    # ax1.set_xlabel('x')
    # ax1.set_ylabel('y')
    # plt.show(block=True)

    return X, z

def calculate_distances(X, X_neg_filtered):
    # Build a KDTree for the positive set
    tree = KDTree(X)

    # Query the tree to find the distance of each point in the negative set to the closest point in the positive set
    distances, _ = tree.query(X_neg_filtered, k=1)

    return distances






