import torch


class BasicDataset(torch.utils.data.Dataset):

    def __init__(self, samples, trg):
        super(BasicDataset, self).__init__()
        self.samples = samples
        self.trg = trg

    def __len__(self):
        return self.samples.shape[0]

    def __getitem__(self, idx):
        # get the sample from the preprocessed sample list and the token stream for faster taget generation
        return [self.samples[idx, :], self.trg[idx, :]]

    def collate_fn(self, insts):
        bsz = len(insts)
        num_in_channels = insts[0][0].shape[0]
        num_out_channels = insts[0][1].shape[0]

        # preallocate final torch tensor that is then written to
        batch_matrix = torch.zeros((bsz, num_in_channels), dtype=torch.float32)
        target = torch.zeros((bsz, num_out_channels), dtype=torch.float32)

        # write directly into the batch
        for inst_idx, inst in enumerate(insts):
            # write batch matrix
            batch_matrix[inst_idx, :] = inst[0]
            # write target matrix
            target[inst_idx, :] = inst[1]
            # no need to mark distance as its zero

        return batch_matrix, target