import numpy as np
from sklearn.metrics import roc_curve, auc
from typing import Literal


def aucroc_multiclass(pred: np.ndarray, trg: np.ndarray, averaging: Literal['weighted', 'unweighted']):
    assert isinstance(pred, np.ndarray)
    assert isinstance(trg, np.ndarray)
    assert len(pred.shape) == 2
    assert len(trg.shape) == 1
    assert pred.shape[0] == trg.shape[0]
    assert averaging in ['weighted', 'unweighted']

    # determine the number of classes
    num_classes = pred.shape[1]

    # compute curves and scores in One-vs-Rest fashion and weighted by prevalence
    weights = []
    scores = []
    curves = []
    for c in range(num_classes):
        probabilities_class_c = pred[:, c]
        true_class_c = trg == c
        fpr, tpr, thresh = roc_curve(true_class_c, probabilities_class_c)
        auc_score = auc(fpr, tpr)
        scores.append(auc_score)
        curves.append([fpr, tpr, thresh])
        weights.append(sum(true_class_c))
    # average
    avg_score = 0.0
    # interpolation_fpr = np.linspace(0, 1, 100)
    # interpolation_tpr = np.zeros_like(interpolation_fpr)
    for c in range(num_classes):
        auc_score = scores[c]
        # fpr, tpr, _ = self.curves[c]
        if averaging == 'weighted':
            weight = weights[c] / sum(weights)
        else:
            weight = 1.0 / num_classes
        avg_score += auc_score * weight
        # interpolation_tpr += np.interp(interpolation_fpr, fpr, tpr) * weight
    avg_score = avg_score
    # self.avg_curve = [interpolation_fpr, interpolation_tpr]

    return avg_score, scores, curves