import numpy as np


def sliding_window_calib_curve(prediction: np.ndarray,
                               target: np.ndarray,
                               bin_size: int):
    # modified from functional_ece using
    # classwise ece probosed by "Beyond temperature scaling:
    # Obtaining well-calibrated multiclass probabilities
    # with Dirichlet calibration"
    # in contrast to Guo's version this uses the prediction of all classes

    # CWECE uses all classes not only the main prediction
    assert len(prediction.shape) == 2
    assert len(target.shape) == 1
    assert target.shape[0] == prediction.shape[0]

    num_points = target.shape[0]

    bin_true_p = []
    bin_pred_p = []
    prediction_probabilities = prediction[:, 1]

    sorted_order = np.argsort(prediction_probabilities)
    prediction_probabilities = prediction_probabilities[sorted_order]
    target = target[sorted_order]

    for i in range(num_points - bin_size):
        bin_pred_p.append(np.mean(prediction_probabilities[i:i+bin_size]))
        bin_true_p.append(np.mean(target[i:i+bin_size]))

    return bin_true_p, bin_pred_p